NUMBER_FOR_REST = 11

CPF = {
    "first_digit": [10, 9, 8, 7, 6, 5, 4, 3, 2],
    "second_digit": [11, 10, 9, 8, 7, 6, 5, 4, 3, 2],
}

CNPJ = {
    "first_digit": [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2],
    "second_digit": [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2],
}


def extract_digits(value):
    return "".join(list(filter(lambda x: x.isdigit(), value)))


def first_digit_cpf_checker_is_valid(value: str):
    combinations = list(zip(CPF.get("first_digit"), [int(x) for x in value[:10]]))
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = 0 if rest_of_division < 2 else NUMBER_FOR_REST - rest_of_division
    return result == int(value[9])


def second_digit_cpf_checker_is_valid(value: str):
    combinations = list(zip(CPF.get("second_digit"), [int(x) for x in value[:11]]))
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = 0 if rest_of_division < 2 else NUMBER_FOR_REST - rest_of_division
    return result == int(value[10])


def first_digit_cnpj_checker_is_valid(value: str):
    combinations = list(zip(CNPJ.get("first_digit"), [int(x) for x in value[:12]]))
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = 0 if rest_of_division < 2 else NUMBER_FOR_REST - rest_of_division
    return result == int(value[12])


def second_digit_cnpj_checker_is_valid(value: str):
    combinations = list(zip(CNPJ.get("second_digit"), [int(x) for x in value[:13]]))
    sum_ = 0
    for item in combinations:
        x, y = item
        sum_ += x * y
    rest_of_division = sum_ % NUMBER_FOR_REST
    result = 0 if rest_of_division < 2 else NUMBER_FOR_REST - rest_of_division
    return result == int(value[13])


def cnpj_or_cpf_is_valid(value: str):
    value = extract_digits(value)
    result = False
    if len(value) == 11 and (
        first_digit_cpf_checker_is_valid(value) and second_digit_cpf_checker_is_valid(value)
    ):
        result = True
    if len(value) == 14 and (
        first_digit_cnpj_checker_is_valid(value) and second_digit_cnpj_checker_is_valid(value)
    ):
        result = True
    return result



"""Setup for pycpf-cnpj.
"""

import os
from setuptools import setup, find_packages


def read(*paths):
    """Read the contents of a text file safely.
    >>> read("pycpf-cnpj", "VERSION")
    '0.1.0'
    >>> read("README.md")
    ...
    """
    root_path = os.path.dirname(__file__)
    filepath = os.path.join(root_path, *paths)
    with open(filepath) as file_:
        return file_.read().strip()


def read_requirements(path):
    """Return a list of requirements from a text file"""
    return [
        line.strip()
        for line in read(path).split("\n")
        if not line.startswith(("#", "git+", '"', "-"))
    ]


setup(
    name="pycnpj-cpf",
    version="0.1.2",
    description="CNPJ and CPF validator",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Carlos Moreno",
    author_email="omorenodovale@gmail.com",
    python_requires=">=3.8",
    packages=find_packages(exclude=["tests", "integration"]),
    include_package_data=True,
    entry_points={"console_scripts": {"pycnpj-cpf = pycnpj_cpf.__main__:main"}},
    install_requires=read_requirements("requirements.txt"),
    extras_require={
        "test": read_requirements("requirements.test.txt"),
        "dev": read_requirements("requirements.dev.txt"),
    },
    url="https://github.com/carlos-moreno/pycnpj-cpf"
)
